/**
 * CSCI 201 Lab 3 (Game).
 * @author CSCI 201 Instructors.
 */
import java.awt.GridLayout;
import java.awt.Frame;
import java.awt.Label;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.Font;

import java.util.ArrayList;
import java.util.Random;
import java.util.Collections;
import java.util.Map;
import java.util.HashMap;

public class Game implements KeyListener {

  // To get Unicode 16 codes, copy emoji from here:
  // https://apps.timwhitlock.info/emoji/tables/unicode
  // and then paste into here and click calculate:
  // https://russellcottrell.com/greek/utilities/SurrogatePairCalculator.htm
  private static final Map<String, String> FOOD_SYMBOLS = createFoodSymbols();
  private static Map<String, String> createFoodSymbols() {
    boolean windows = isWindows();
    Map<String, String> result = new HashMap<>();
    result.put("apple", windows ? "a" : "\uD83C\uDF4E");
    result.put("mushroom", windows ? "m" : "\uD83C\uDF44");
    result.put("pumpkin", windows ? "p" : "\uD83C\uDF83");
    result.put("strawberry", windows ? "s" : "\uD83C\uDF53");
    result.put("tomato", windows ? "t" : "\uD83C\uDF45");
    result.put("corn", windows ? "c " : "\uD83C\uDF3D");
    result.put("sunflower", windows ? "f" : "\uD83C\uDF3B");
    return result;
  }
  public static boolean isWindows() {
     return System.getProperty("os.name").startsWith("Windows");
  }
  private static final String PLAYER_SYMBOL = isWindows() ? "B" : "\uD83D\uDC07"; // bunny
  private static final String DOG_SYMBOL = isWindows() ? "D" : "\uD83D\uDC36"; // dog

  // Properties of the game display.
  public static final int NROWS = 10;
  public static final int NCOLS = 10;
  private static final int WIDTH = 500;
  private static final int HEIGHT = 500;
  private static final int FONT_SIZE = 25;

  // Frame and labels used to display the game.
  private Frame frame;
  private Label[][] labels;

  // game pieces
  private Player player;
  private Dog dog;
  private ArrayList<Food> food;

  /**
   * Utility to create an array of random food.
   * 
   * @param nFood - number of pieces of food to create
   * @return ArrayList of food
   */
  public static ArrayList<Food> createFood(int nFood) {
    ArrayList<Food> food = new ArrayList<Food>();
    Random randFood = new Random();
    ArrayList<Integer> positions = new ArrayList<Integer>();
    for (int i = 1; i < NROWS * NCOLS; i++) {
      positions.add(i);
    }
    Collections.shuffle(positions);
    String[] foodNames = FOOD_SYMBOLS.keySet().toArray(new String[0]);
    for (int k = 0; k < nFood; k++) {
      int index = positions.get(k);
      int i = index / NROWS;
      int j = index % NCOLS;
      int f = randFood.nextInt(foodNames.length);
      food.add(new Food(foodNames[f], i, j));
    }
    return food;
  }

  /**
   * Initializes the game display and pieces.
   */
  public Game() {
    // Create the empty NROWS x NCOLS frame.
    frame = new Frame("Lab 3");
    frame.setLayout(new GridLayout(NROWS, NCOLS, 0, 0));
    frame.setSize(WIDTH, HEIGHT);
    frame.setVisible(true);
    frame.validate();
    labels = new Label[NROWS][NCOLS];
    for (int i = 0; i < NROWS; i++) {
      for (int j = 0; j < NCOLS; j++) {
        labels[i][j] = new Label(" ", Label.CENTER);
        labels[i][j].setFont(new Font("Calibri", Font.PLAIN, FONT_SIZE));
        frame.add(labels[i][j]);
      }
    }

    // Initialize player, human and food.
    player = new Player(0, 0);
    dog = new Dog(NCOLS / 2, NROWS / 2);
    food = createFood(10);
  }

  /**
   * Starts the game and draws the initial state.
   */
  public void start() {
    frame.addKeyListener(this);
    draw();
  }

  /**
   * Listens for events in which the user types.
   * This isn't used but needs to be implemented
   * for the KeyListener interface.
   */
  @Override
  public void keyTyped(KeyEvent e) {
  }

  /**
   * Listens for a key press.
   * If a user presses 'q', the game ends.
   */
  @Override
  public void keyPressed(KeyEvent e) {
    if (e.getKeyChar() == 'q') {
      // Print the player's score.
      System.out.println("Score = " + player.getScore());
      System.exit(0);
    }
  }

  /**
   * Listens for a key released.
   * This moves the player and the dog, and
   * checks for any updates to the game state.
   */
  @Override
  public void keyReleased(KeyEvent e) {
    int code = e.getKeyCode();
    if (code == KeyEvent.VK_UP) {
      player.y--;
    } else if (code == KeyEvent.VK_DOWN) {
      player.y++;
    } else if (code == KeyEvent.VK_RIGHT) {
      player.x++;
    } else if (code == KeyEvent.VK_LEFT) {
      player.x--;
    } else {
      return;
    }
    dog.moveRandomly();
    dog.limit();
    player.limit();
    player.checkFood(food);
    player.checkDog(dog, food);
    draw();
  }

  /**
   * Draws the current game state.
   */
  private void draw() {
    for (int i = 0; i < NROWS; i++) {
      for (int j = 0; j < NCOLS; j++) {
        labels[i][j].setText(" ");
      }
    }
    for (Food f : food) {
      labels[f.y][f.x].setText(FOOD_SYMBOLS.get(f.getName()));
    }
    labels[player.y][player.x].setText(PLAYER_SYMBOL);
    labels[dog.y][dog.x].setText(DOG_SYMBOL);
    frame.revalidate();
  }

  /**
   * Starts the game!
   * 
   * @param args - array of String arguments (not used).
   */
  public static void main(String[] args) {
    Game game = new Game();
    game.start();
  }
}
